// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef HK_COLUMN
#define HK_COLUMN
#include <hk_class.h>
#include <list>
class hk_datasource;
class hk_dsdatavisible;
/**
*
*@short represents one column of a datasource.
*@version $Revision: 1.16 $
*@author Horst Knorr (hk_classes@knorrnet.de)
*
*represents one column of a datasource. Values of the current row (set in @ref hk_datasource)
*can be altered by using the functions @ref set_asstring, @ref set_asbool and @ref set_asblob
*and read by using the functions @ref asstring, @ref asbool and @ref asblob.
*
*If the the datasource is in alter or create mode you can change the column definition by using the following
*functions: @ref set_name, @ref set_fieldtype, @ref set_size, @ref set_notnull, @ref set_primary

*/

class hk_column:public hk_class

{
       friend class hk_datasource;
       friend class hk_dsdatavisible;
public:
/**
*name of the column
*/
	   hk_string	name(void);
/**
*name of the column. Can just be edited if the datasource is in the mode ALTER or CREATE.
*/
		void	set_name(const hk_string& n);

/**
* @return true if data in this column can be changed.
* Always false if the datasource is a dml_query.

*/
		bool	is_readonly(void);
/**
* If the datasource is of type ds_table you can allow or disallow data changes in this column.
*/
		void	set_readonly(bool r);

/**
*used from hk_column to define of what type it is.
*For details how the different field types are interpreted
* look up the documentation of your database server and the database driver.
*
*textfield    : fixed length (usually in SQL defined as CHAR(size))
*
*auto_incfield:	if the databasedriver supports this fieldtype it will get a unique number automatically, whenever
*		a new row will be added. Usually a readonly column
*
*smallintegerfield: a integer range of at least -32768 to 32767
*
*integerfield: 	a integer with the maximum range the database server can handle
*
*smallfloatingfield: a floating field with a range of at least ????
*
*floatingfield: a floating field with the maximum range the database server can handle
*
*datefield:    a field to keep the day,month and year of a date
*
*datetimefield: a field to keep the day,month and year of a date PLUS the hour,minute and second of a time
*
*timefield:    a field to keep the hour,minute and second of a time.
*
*binaryfield:	a field to store binary data of more or less any size.
*
*memofield:	a field to store text data of more or less any size.
*
*boolfield:	a field to 'true' or 'false'
*
*otherfield:	a field hk_classes does not know, but tries to handle as good as it can.

*/
enum enum_fieldtype {textfield,auto_incfield,smallintegerfield,integerfield,smallfloatingfield,floatingfield,datefield,datetimefield,timefield,binaryfield,memofield,boolfield,otherfield};

/**
*
*If you need the field type as a hk_string use @ref fieldtype_name to translate.
*@return  the type of the field of type @ref enum_fieldtype.
*/
	enum_fieldtype 	fieldtype(void);
/**
* can just be edited if the datasource is in the mode ALTER or CREATE.
*
*/
		void	set_fieldtype(enum_fieldtype f);
/**
*field length in bytes
*/
		long 	size(void);
/**
* can just be edited if the datasource is in the mode ALTER or CREATE.
*/
		void	set_size(long s);
/**
* to translate the fieldtype into a hk_string.
*/
	hk_string	fieldtype_name(void);
/**
*@return true if this field is part of the primary key (primary index).
*/
		bool	is_primary(void);
/**
* can just be edited if the datasource is in the mode ALTER or CREATE.
*/
		bool	set_primary(bool i);
/**
*@return  true if this field has to have a value.
*/
		bool	is_notnull(void);
/**
* can just be edited if the datasource is in the mode ALTER or CREATE.
*/
		bool	set_notnull(bool i);
/**
*@return true if the value of the field has changed and not yet saved in the database.
*/
		bool	has_changed(void);
/**
*returns the value of this field, type converted to a hk_string. (All fieldtypes except BLOB).
*/
	hk_string 	asstring(void);
/**
* @param s new value of the field.
*/
		void	set_asstring(const hk_string& s,bool registerchange=true);
/**
* Fields can contain characters such as \0, \n, ' and so on.
* Some SQL Servers need to encode this data, so this function delivers the encoded version.
*/
	const 	char*	transformed_asstring(void);
	const 	char*	transformed_asstring_at(unsigned long position);
/**
*if new data was inserted with the method @ref asstring and the
*changes have not yet been stored this method returns the changed data
*/
	hk_string	changed_data_asstring(void);
/**
*changes the value of the field in native form.(i.e. Binary data)
*@see load_from_file
*/

	void	set_asblob(const struct_raw_data* rd);
/**
*returns the value of the field as raw data.
*@see save_to_file.
*/
	const	struct_raw_data* asblob(void);
/**
*returns the value of the field at a specific rowposition as raw data.
*/
	const	struct_raw_data* asblob_at(unsigned long a);

/**
* @param b new value of the field.
*/
		void 	set_asbool(bool b);
/**
*returns the value of this field, type converted to a bool. (All fieldtypes except BLOB).
*/
		bool 	asbool(void);
		bool	asbool_at(unsigned long a);
		bool	changed_data_asbool(void);

/**
*In a SQL statement field values may be delimited by a "  which might be driver specific.
*(i.e. SELECT * FROM test WHERE myname= "Knorr" ). For internal use only.
*/
	hk_string	sql_column_delimiter(void);
/**
* A temporarily given unique number of this row
*/
	unsigned int 	fieldnumber(void);

/**
*returns the value at a specific position in the column. The row selector will not be changed,
*so the visible objects will not be informed. Use @ref asstring in combination with
*@ref hk_datasource::goto_row instead.
*/
	hk_string	asstring_at(unsigned long a);
/**
*search for a specific value in a column
*@param from number of the first row where the search has to start. First row is number 0
*@param to number of the last row where the search has to end. Last row is number max_rows-1;
*@param searchtext value as hk_string to look for
*@param wholephrase if false the searchtext is part of columnvalue, if true it has to be an exact match
*@param casesensitive if true a case sensitive compararation is used
*@param backwards  searching backwards, default is forward
*@return row number if found, max_rows()+1 if not found
*/
unsigned int find(unsigned int from,unsigned int to,const hk_string& searchtext,bool wholephrase=false,bool casesensitive=false,bool backwards=false);
/**
* search from the beginning to the end for searchtext
@param searchtext value as hk_string to look for
*@return row number if found, max_rows()+1 if not found
*/
unsigned int find(const hk_string& searchtext,bool wholephrase=false,bool casesensitive=false,bool backwards=false);
/**
*an internal used function which compares the column value at row number "pos"
*/
bool is_findstring(unsigned int pos,const hk_string& searchtext,bool wholephrase=false,bool casesensitive=false);

/**
*returns the new data set by a function like asstring(hk_string& n);
*/
	const 	struct_raw_data*	changed_data(void);

/**
*returns the new data set by a function like asstring(hk_string& n) in SQL server native form.
*/

	const	struct_raw_data*	transformed_changed_data(void);
/**
*set all changed data back to its default value. Usually you dont
*have to call this function
*/
		void	reset_changed_data(void);

/**
* you have the possibility to set the values which the column will accept as
* true and false. i.e. 0 and 1 for numeric fields or for text fields yes and no,
* or true and false.
*/
	void set_boolvalues(const hk_string& TRUE, const hk_string& FALSE);
	hk_string bool_truevalue(void);
	hk_string bool_falsevalue(void);
/**
* if the column is of type date, time or datetime you can specify the wished
* format with this function. See @ref hk_datetime for details.
*/
	void set_dateformat(const hk_string& df){p_dateformat=df;}
/**
* if the column is of type date, time or datetime you can specify the wished
* format with this function. See @ref hk_datetime for details.
*/
	void set_timeformat(const hk_string& tf){p_timeformat=tf;}
/**
* if the column is of type date, time or datetime you can specify the wished
* format with this function. See @ref hk_datetime for details.
*/
	void set_datetimeformat(const hk_string& dtf){p_datetimeformat=dtf;}
/**
* save the value of this field in a file. Very useful for blob-fields
*/
	bool save_to_file(const hk_string& filename);
/**
*changes the value of the field with the content of the file"filename". Very useful
*for blob fields.
*/
	bool load_from_file(const hk_string& filename);
protected:
			hk_column(hk_datasource* ds, const hk_string& TRUE,const hk_string& FALSE);
		virtual	~hk_column();
//	virtual	hk_string driver_specific_asstring(void){return "";}
	virtual bool	    driver_specific_asstring(const hk_string& d);
	virtual hk_string driver_specific_asstring_at(unsigned long a);
	virtual const char* driver_specific_transformed_asstring_at(unsigned long ){return "";}
	virtual	const struct_raw_data* driver_specific_asblob(void){return NULL;}
	virtual const struct_raw_data* driver_specific_asblob_at(unsigned long a);
	virtual bool	    driver_specific_asblob(const char* b);
	virtual void 	    driver_specific_asbool(bool b);
	virtual bool	    driver_specific_asbool(void){return false;}
	virtual bool	    driver_specific_asbool_at(unsigned long a);
	virtual void data_has_changed(void);
	void datavisible_add(hk_dsdatavisible*);
	void datavisible_remove(hk_dsdatavisible*);
        hk_datasource*	p_datasource;
        unsigned int 	p_fieldnr;
 	bool		p_readonly;
        unsigned long	p_driver_specific_data_size;
        char*		p_driver_specific_data;
        char*		p_original_new_data; // where datachanges are stored temporarily
        unsigned long	p_original_new_data_size;
        hk_string		p_new_data_asstring;
	hk_string		p_true;
	hk_string		p_false;
        hk_string 		p_dateformat;
        hk_string		p_timeformat;
        hk_string		p_datetimeformat;
        hk_string 		p_driverspecific_dateformat;
        hk_string		p_driverspecific_timeformat;
        hk_string		p_driverspecific_datetimeformat;

private:
	hk_string 		p_fieldname;
	enum_fieldtype	p_fieldtype;
        long 		p_size;
        bool		p_primary_index;
        bool		p_notnull;
        bool 		p_has_changed;
        hk_string		p_sql_column_delimiter;
        struct_raw_data p_blob;
        struct_raw_data p_changed_data_struct;
        struct_raw_data p_transformed_changed_data_struct;
	list<hk_dsdatavisible*> p_datavisibles;
	void set_has_changed();
	void set_has_not_changed();
	hk_internal* p_internal;

} ;


#endif
