// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef HK_CONNECTION
#define HK_CONNECTION
//#include <hk_string>
#include <hk_class.h>
#include <list>
#include <vector>
#include <algorithm>
class hk_drivermanager;

class hk_database;
/**
*@short connects to the SQL Server
*@version $Revision: 1.8 $
*@author Horst Knorr (hk_classes@knorrnet.de)
*
*connects to the SQL Server. Set @ref set_host,@ref set_user, @ref set_password ,
*@ref set_tcp_port if necessary and then call @ref connect.

*/
class hk_connection:public hk_class
{
  friend class hk_database;
  friend class hk_drivermanager;
 public:
/**
* When the connection will be destroyed, the database created with @ref new_database() and all
* its children will be destroyed
**/
virtual	~hk_connection();
/**
* name of the host where the SQL Server is. Default is "localhost".
*/
 	void 	set_host(const hk_string& h);
hk_string 	host(void);
/**
* user name for the SQL Server.
*/
 	void 	set_user(const hk_string& u);
hk_string 	user(void);
/**
* password for the SQL Server
*/
 	void   	set_password(const hk_string& p);
hk_string  	password(void);
/**
*  TCP Port number of the SQL Server. Default is 3306.
*/
 	void 		set_tcp_port(unsigned int t);
 	unsigned int 	tcp_port(void);
/**
* tries to connect to the SQL Server. If some information is missing (i.e. password)
* a dialog will ask for these values. You can set a GUI dependend dialog with @ref set_passworddialog
* @return true if connecting was successful, else false.
*/
	bool	connect(void);
/**
* disconnects from the SQL Server
* @return true if successful.
*/
	bool	disconnect(void);
/**
*@return true  if @ref connect was successful
*/
	bool	is_connected(void);
/**
*if connected it returns a hk_stringlist of all existing databases on the SQL Server.
*The list will be created new every time you call this function.
*So be careful. Never use i.e. for_each(dblist()->begin(),dblist()->end(),anyfunction)
*Use instead:
*list<hk_string>* mylist=dblist();
*for_each(mylist->begin(),mylist->end(),anyfunction)

*/
const	vector<hk_string>* dblist(void);
/**
* if you want to use a database on the SQL Server create a hk_database element
* with this class. For each connection you can define one database. If you define another
* one the old one will be destroyed.

*/
	hk_database* new_database(const hk_string& name="");
/**
*give information about the database server. See @hk_definitions for details
*/
virtual int server_supports(void){return supports_sql;}
/**
*shows what information the database server needs (i.e. password). See @hk_definitions for details
*/
virtual int server_needs(void){return needs_password;}
/**
* the passworddialog will be called from @ref connect if necessary. You can define a GUI dependend
*dialog and set it with this function. What exactly is asked in the dialog is database driver dependend
*(i.e. the Mysql driver needs user, password, host and tcp_port)
*/
static	void 	set_passworddialog(password_dialogtype*);
/**
* tries to create a new database. If successful it returns true else false
*/
bool	create_database(const hk_string& dbase);
/**
* tries to delete a database.
*/
bool	delete_database(const hk_string& dbase);
 protected:
 	hk_connection(hk_drivermanager* c);
	bool 	p_connected;
virtual	bool	driver_specific_connect(void){return false;}
virtual	bool	driver_specific_disconnect(void){return false;}
virtual const vector<hk_string>* driver_specific_dblist(void){return NULL;}
virtual hk_database* driver_specific_new_database(void){return NULL;}
	void	db_remove(hk_database* d);
	hk_string sqldelimiter(void) {return p_sqldelimiter;}
	void inform_databases_connect(void);
 	vector<hk_string>* p_databaselist;
	bool show_passworddialog(void);
/**
*saves the tags <HOST><USER><TCP-PORT><SQLDELIMITER> with the equivalent values
*/
virtual	void savedata(ofstream& s);
/**
*load the tags <HOST><USER><TCP-PORT><SQLDELIMITER> with the equivalent values
*/
virtual void loaddata(const hk_string& definition);
 private:
 void load_configuration(void);
 void save_configuration(void);
hk_string databasepath(void){return p_databasepath;}
       void set_classespath(hk_string& p);
	hk_string 	p_host,p_user,p_password;
 	unsigned int 	p_tcp_port;
        hk_database* p_database;
        unsigned int p_debug_counter;
        bool p_connection_successfully_tried;
static  password_dialogtype* p_passworddialog;
        hk_string p_sqldelimiter;
	hk_string p_classespath;
	hk_string p_databasepath;
	hk_drivermanager* p_drivermanager;

} ;


#endif
