// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef HK_REPORTDATA
#define HK_REPORTDATA
#include "hk_definitions.h"
#include "hk_dsdatavisible.h"
#include <list>
#include <vector>
//#include <hk_string>
#include <iostream>
#include <fstream>
#include <time.h>

class hk_reportsection;
class hk_reportdata;
class hk_report;


/**
* represents the data of a field within a section
* @short represents the data of a field within a section.
*@version $Revision: 1.5 $
* @author Horst Knorr (hk_classes@knorrnet.de)

*
*In addition to the global variables defined in @ref hk_reportbase you can use the columnnames delimited by #
*as variablenames. If a @ref hk_report::recodefunction is set, the values will be recoded.
*<pre>
%PAGENUMBER%     the number of the page
%ABSOLUTEPAGENUMBER%     physical number of the page in file
%ROWNUMBER%      the actual row number of the datasource

#column#         the value of the column as shown in the following two examples
#field1#         the value of the column with the name field1
#first_name#     the value of the column with the name first_name
</pre>
*If you have set @ref set_columnname  you have additionally the following variable names
<pre>
%value%          the raw value of the column defined with @ref set_columnname
%fieldname%      the name of the column not recoded defined with set_columnname
%VALUE%          the value of the column defined with @ref set_columnname recoded
                 with @ref hk_report::set_recodefunction
%FIELDNAME%      the recoded (with @ref hk_report::set_recodefunction ) name of the column
                 defined with @ref set_columnname
%COUNT%          the number of values
</pre>
*If the column is of type integer or real you have the following variables
<pre>
%SUM%            the sum of all values
%MIN%            the minimum value
%MAX%            the maximum value
%STDDEV%         standard deviation
%STDDEVSAMPLE%   standard deviation of a sample
%AVERAGE%        the average of all values
</pre>
*/

class hk_reportdata :public hk_dsdatavisible
{
	friend class hk_reportsection;
	public:
/**
*Set the data that should be printed when the section will be printed
*You can use all of the above named variables.
*Usually you connect a report_data part with a column via @ref set_columnname
*The most easy way is to set the data to %VALUE%.
* A more intermediate way will be shown in the following example
*
*Example:
*
*You want to automatically create links in a html report. For this reason
*you have a datasource with 2 columns "url", and "name". "url" has the URL, "name" contains a description
i.e.
*<pre>
URL			name
http://www.suse.de 	SuseLinux
http://www.redhat.com	RedhatLinux
http://www.debian.org	Debian
*</pre>
To create a html link use set_data("<A HREF=\"#url#\">#name#</A>");
*/
	void set_data(const hk_string& d){p_data=d;}
	hk_string data(void){return p_data;}
	void set_beforedata(const hk_string& b){p_beforedata=b;}
	void set_afterdata(const hk_string& a){p_afterdata=a;}
/**
*The name of a column of the datasource. Will be used for some variables such as %COUNT%,
*%SUM%,AVERAGE% and so on.
*/
/**
*returns the value that will be printed in the section
*/
        hk_string actual_hk_string(void);
/**
* Will be internally called to create the values for the variables %COUNT%,%SUM% etc.
*/
	void count(void);
/**
*The values of the variables %SUM% %COUNT% %AVERAGE% %STDDEV% etc. can be relatively to the section.
*i.e. if the section is a uniquesection and it changes all variables can be reset to 0 (default).
* @param c if true these variables will change to global variables in the report and not be set to 0 when
*the section changes.
*/
	void set_runningcount(bool c=true){p_runningcount=c;}
/**
*@see hk_reportsection::set_replacefunction
*/
void		set_replacefunction(data_replacefunctiontype* f){p_replacefunction=f;}
virtual	hk_string font(void);
virtual hk_string fontsize(void);
/**
*Defines whether the value should be aligned left, right or center.
*/
	void set_alignment(alignmenttype a=alignleft){p_align=a;}
alignmenttype alignment(void){return p_align;}
/**
*needed by @ref hk_report::set_periodic. It is possible to compute as how much a section counts.
*@param f is a userdefined function that will be called whenever @ref counts_as is called.
*/
        void set_datacountfunction(reportdatacounttype* f){p_datacountfunction=f;}
        reportdatacounttype* datacountfunction(void){return p_datacountfunction;}
/**
*if true a line should be painted above the data
*
*See @ref set_configurefunction  for details
*
*/
void	set_topline(bool l=true);
/**
*if true a line should be painted under the data
*
*See @ref set_configurefunction  for details
*/
void	set_bottomline(bool l=true);
/**
*if true a line should be painted left of the data
*
*See @ref set_configurefunction  for details
*/

void	set_leftline(bool l=true);
/**
*if true a line should be painted right the data
*
*See @ref set_configurefunction  for details
*/

void	set_rightline(bool l=true);
/**
*if true a diagonal should be painted from _l_inks _u_nten to _r_echts _o_ben
* (left down to right up)
*
*See @ref set_configurefunction  for details
*/

void	set_diagonalluro(bool l=true);
/**
*if true a diagonal should be painted from _l_inks _o_ben to _r_echts _u_nten
* (left top to right down)
*
*See @ref set_configurefunction  for details
*/
void	set_diagonalloru(bool l=true);
bool	diagonalloru(void){return p_loru;}
bool	topline(void){return p_top;}
bool	bottomline(void){return p_bottom;}
bool	leftline(void){return p_left;}
bool	rightline(void){return p_right;}
bool	diagonalluro(void){return p_luro;}
/**
*if true a frame should be painted around the text (bottomline + topline + leftline + rightline)
*
*See @ref set_configurefunction  for details
*/
void	set_frame(bool l=true);
/**
*if true the textline should be broken if the line is longer than the width
*
*See @ref set_configurefunction  for details
*/
void	set_wordbreak(bool b=true);
bool	wordbreak(void){return p_wordbreak;}
/**
*in this function you have to define how to react on i.e. set_bottomline etc.
*/
void	set_configurefunction(data_configurefunctiontype* f);
virtual void 	savedata(ofstream& s);
virtual void  loaddata(const hk_string& definition);

	protected:
		hk_reportdata(hk_report*,hk_reportsection*);
		~hk_reportdata(void);
	void reset_count();
virtual hk_string replace(const hk_string& where);
	bool p_top;
	bool p_left;
	bool p_right;
	bool p_bottom;
	bool p_loru;
	bool p_luro;
	bool p_wordbreak;
	private:
	hk_string p_data;
	hk_string p_beforedata;
	hk_string p_afterdata;
	hk_string p_displayname;
        hk_report* p_report;
	hk_reportsection* p_section;
        unsigned long p_count;
        bool p_runningcount;
        bool p_minmax_alreadyset;
        number sum;
        number min;
        number max;
        longnumber squaresum;
        unsigned int p_fillchars;
        data_replacefunctiontype* p_replacefunction;
        alignmenttype p_align;
        reportdatacounttype* p_datacountfunction;
	data_configurefunctiontype* p_dataconfigurefunction;
};

#endif
