// ****************************************************************************
// copyright (c) 2000,2001 Horst Knorr <hk_classes@knorrnet.de>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef HK_REPORTSECTION
#define HK_REPORTSECTION
#include "hk_definitions.h"
#include "hk_dsdatavisible.h"
#include <list>
#include <vector>
//#include <hk_string>
#include <iostream>
#include <fstream>
#include <time.h>
class hk_reportsection;
class hk_reportdata;
class hk_report;
class hk_column;
typedef unsigned long reportsectioncounttype(hk_reportsection*);
/**
*You can define your own functions for sections. See @ref hk_reportsection::set_replacefunction for details.
the function will be called after replacing the built-in variables and will give the "this"-Pointer and the text
*that would be printed as parameters. Your function has to return a hk_string that will be printed instead.
*/
typedef hk_string	section_replacefunctiontype(hk_reportsection*,const hk_string& );



typedef struct
{
  number sum;
  number min;
  number max;
  number value;
  unsigned long count;
  longnumber squaresum;
  char colname[75];
//  enum_fieldtype fieldtype;
} struct_countnumbers;

/**
* hk_reportsection represents a section in a hk_report element. Within a section
* you can define the outputdata,use it as headers for your data and so in
*If it is needed as a section header or footer use @ref set_unique in combination with @ref set_uniquecolumn
*It also can contain subreports
*
*<pre>Example: Defining a CVS output (no first row with columnnames)
* //
* // first define a datasource
* //
*hk_classes p_class;
*hk_connection* connect=p_class.hk_create("mysql");
*connect->user("root");
*connect->host("localhost");
*connect->password("my_password");
*connect->connect();
*hk_database* database=connect->new_database();
*database->name("vokabeltrainer");
*hk_datasource* ds=database->new_table();
*ds->name("vokabeln");
* //
* //now start defining the report
* //
*hk_report e;
*e.datasource(ds);
*hk_reportsection* s=e.new_section();
*s->set_default_columnvalue("\"%VALUE%\""); // define " before and after the value of a column
*s->set_betweendata(" , "); // a comma separated list
*s->set_sectionend("\n"); // one row per datarow
*e.execute();
*</pre>
* @short represents a section of the report.
*@version $Revision: 1.4 $
* @author Horst Knorr (hk_classes@knorrnet.de)
*/

class hk_reportsection : public hk_dsdatavisible
{
friend class hk_report;
	public:
/**
*If the section should be printed just once per block and not for each datarow use this function
*@param unique true if this section should be printed once
*@param fieldname Name of a column. If the column changes its value then this unique section will be printed
*interesting four grouped data
*@param endsection true if this section should not be printed as a header. Instead it will be printed at the end of the block
*/
void		set_unique(bool unique=true, const hk_string& fieldname="", bool endsection=false);
hk_string		unique_column(void){return p_uniquecolumnname;}
bool		unique(void){return p_unique;}
/**
*the printed data. See @ref hk_reportdata and @ref default_columnvalue for details
*/
hk_reportdata*  new_data(void);
hk_reportdata*  new_data_at(unsigned int);
hk_reportdata*	data_at(unsigned int);
//void		delete_data(hk_reportdata*);
//void		delete_at(unsigned int);
vector<hk_reportdata*>* datalist(void) {return &p_data;}
/**
*will send the actual data to the outputstream  @ref hk_report::outputstream
*/
virtual void 		actual_hk_string(void);
/**
*The sectionbegin will be printed as a header of the section (before the data)
*/
hk_string		sectionbegin(void){return p_sectionbegin;}
void		set_sectionbegin(const hk_string& b){p_sectionbegin=b;}
/**
*The sectionend will be printed as a footer of the section (after the data)
*i.e. a \n at the end of a datarow
*/
hk_string		sectionend(void){return p_sectionend;}
void		set_sectionend(const hk_string& e){p_sectionend=e;}
/**
*A hk_string that will be printed between two datasegments defined with @ref new_data
*i.e. If you want a comma separated list you would need set_betweendata(" , ");
*/
hk_string 		betweendata(void){return p_betweendata;}
void		set_betweendata(const hk_string& b){p_betweendata=b;}
/**
*@return true if this section is a unique section and this section has to be printed
*/
bool		new_uniquevalue(void);
/**
* @return true if this section is a unique section and will be printed behind the data
*/
bool		endsection(void){return p_uniqueendsection;}
/**
*the default column value will be used as a default when a new data will be created with @ref new_data
*default is @VALUE@
*/
hk_string		default_columnvalue(void){return p_default_columnvalue;}
void		set_default_columnvalue(const hk_string& v){p_default_columnvalue=v;}
hk_string		default_beforecolumnvalue(void){return p_default_beforecolumnvalue;}
void		set_default_beforecolumnvalue(const hk_string& v){p_default_beforecolumnvalue=v;}
hk_string		default_aftercolumnvalue(void){return p_default_aftercolumnvalue;}
void		set_default_aftercolumnvalue(const hk_string& v){p_default_aftercolumnvalue=v;}
void	        set_defaultdataconfigurefunction(data_configurefunctiontype* f){p_default_dataconfigurefunction=f;}

/**
*If set true and you havent defined any data with @ref new_data it will automatically create the data
*set with @ref default_columnvalue
*/
void 		set_automatic_create_data(bool d=true){p_automatic_create_datas=d;}
/**
*true if a new page will be started after printing this section. Usually this function should only be used
*in combination with @ref set_unique as an  endsection
*/
bool		new_page_after_section(void){return p_newpageaftersection;}
void		set_new_page_after_section(bool p){p_newpageaftersection=p;}
/**
* @return true if this section has been printed (unique sections will not be printed for every row) else false
*/
bool		section_was_printed(void){return p_sectionwasprinted;}
/**
*A complete other report can be printed within a section
*@param subreport the subreport
*@param before_data if true the subreport will be printed before the data of this section will be printed, else
*it will be printed after the data
*/
void		set_subreport(hk_report* subreport=NULL,bool before_data=false);
bool		print_subreport_before_data(void){return p_print_subreport_before_data;}
void		print_subreport(void);
/**
*You can enhance the possibilities of the section by writing your own function.This function could for example handle your
*own variables.
*
*/
void		set_replacefunction(section_replacefunctiontype* f){p_replacefunction=f;}
virtual hk_string replace(const hk_string& where);
/**
*Will set all COUNT, SUM, AVERAGE and so on variables to 0, if they are not "running counts" (which mean global
*for the whole report).See @ref hk_reportdata for details. Usually you dontt have to call this function
*/
void		reset_count(void);
virtual	hk_string font(void);
virtual hk_string fontsize(void);
virtual unsigned long counts_as(void);
/**
*needed by @ref hk_report::set_periodic. It is possible to compute as how much a section counts.
*@param f is a userdefined function that will be called whenever @ref counts_as is called.
*/
        void set_sectioncountfunction(reportsectioncounttype* f){p_sectioncountfunction=f;}
        reportsectioncounttype* sectioncountfunction(void){return p_sectioncountfunction;}
virtual void  savedata(ofstream& s);
virtual void  loaddata(const hk_string& definition);
virtual void set_datasource(hk_datasource* d);


	protected:
		hk_reportsection(hk_report*);
		~hk_reportsection(void);
	void	automatic_create_datafields(void);
	virtual void	reset(void);
        bool p_already_automaticcreated_data;

        vector<hk_reportdata*> p_data;
        hk_string 	p_uniquecolumnname;
        bool   p_unique;
        hk_string p_lastuniquevalue;
        hk_string p_sectionbegin;
        hk_string p_sectionend;
        hk_string p_betweendata;
        hk_report* p_report;
        hk_report* p_subreport;
        bool	   p_print_subreport_before_data;
        bool p_uniqueendsection;
        bool p_automatic_create_datas;
        hk_string p_default_columnvalue;
        hk_string p_default_beforecolumnvalue;
        hk_string p_default_aftercolumnvalue;
        data_configurefunctiontype* p_default_dataconfigurefunction;
        bool p_newpageaftersection;
        bool p_sectionwasprinted;
        section_replacefunctiontype* p_replacefunction;
	private:
	void create_countingfields(void);
	void count_countingfields(void);
	void clear_countingfields(void);
	reportsectioncounttype* p_sectioncountfunction;
	hk_datasource* p_subreportdatasource;
	bool p_automatic_created_datasource;
	vector<struct_countnumbers> p_countings;

};

#endif
